<?php
/**
* NOTICE OF LICENSE
*
* This source file is subject to the Open Software License (OSL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/osl-3.0.php
*
*  @author    EveryPay
*  @copyright since 2015 EveryPay
*  @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
*/

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

if (!defined('_PS_VERSION_')) {
    exit;
}

class EveryPay extends PaymentModule
{
    const ORDER_BEFORE = 'before';
    const ORDER_AFTER = 'after';

    const THEME_LIGHT = 'light';
    const THEME_DARK = 'dark';

    const LAYOUT_TAB = 'tab';
    const LAYOUT_ACCORDION = 'accordion';

    const COUNTRY_DEFAULT = 'default';
    const COUNTRY_ESTONIA = 'EE';
    const COUNTRY_LATVIA  = 'LV';
    const COUNTRY_LITHUANIA = 'LT';

    const RETURN_SUCCESS = 'completed';
    const RETURN_CANCEL = 'voided';
    const RETURN_FAILED = 'failed';
    const RETURN_SETTLED = 'settled';
    const RETURN_PROCESSING = 'sent_for_processing';
    const RETURN_INVALID_AMOUNT = 'invalid_amount';

    const DEBUG_LOGGING = 0;

    const PAYMENT_STATUS_TABLE = 'everypay_payment_status';

    const BASE_PROD_URL = 'https://pay.every-pay.eu/';
    const BASE_TEST_URL = 'https://igw-demo.every-pay.com/';

    const ONE_OFF_TEST_URL = self::BASE_TEST_URL . 'api/v3/payments/oneoff';
    const ONE_OFF_PROD_URL = self::BASE_PROD_URL . 'api/v3/payments/oneoff';

    const PAYMENT_ELEMENT_TEST_URL = self::BASE_TEST_URL . 'payment_elements/everypay-sdk-v1-0-0.umd.js';
    const PAYMENT_ELEMENT_PROD_URL = self::BASE_PROD_URL . 'payment_elements/everypay-sdk-v1-0-0.umd.js';

    public $os_processing = array();
    public $os_awaiting = array();

    public function __construct()
    {
        $this->name = 'everypay';
        $this->tab = 'payments_gateways';
        $this->author = 'EveryPay';
        $this->version = '8.0.0';
        $this->is_eu_compatible = 1;
        $this->bootstrap = true;
        $this->controllers = array('mycards');
        $this->displayName = $this->l('EveryPay');
        $this->description = $this->l(
            'Accept popular payment methods with a seamless checkout experience – powered by EveryPay.'
        );
        $this->confirmUninstall = $this->l('Are you sure to uninstall EveryPay?');
        $this->ps_versions_compliancy = array('min' => '1.7.0.0', 'max' => _PS_VERSION_);
        $this->module_key = 'db60dd5f7405f17e07daff841db0bb14';
        $this->os_processing = array(
            'status_name' => 'PS_OS_EVERYPAY_PROCESSING',
            'description' => sprintf('Sent for processing (%s)', $this->displayName),
            'descriptionEE' => sprintf('Makse töötlemisel (%s)', $this->displayName),
            'color' => "#fde4ba"
        );
        $this->os_awaiting = array(
            'status_name' => 'PS_OS_EVERYPAY',
            'description' => sprintf('Awaiting %s payment', $this->displayName),
            'descriptionEE' => sprintf('Ootab %s maksmist', $this->displayName),
            'color' => "#fde4ba"
        );

        if (!class_exists('DateHelper')) {
            include $this->local_path . 'lib/DateHelper.php';
        }

        parent::__construct();
    }

    public function install()
    {
        Configuration::updateValue($this->formatName('demo_url'), 'https://igw-demo.every-pay.com/api/v3/payments/');
        Configuration::updateValue($this->formatName('production_url'), 'https://pay.every-pay.eu/api/v3/payments/');
        Configuration::updateValue($this->formatName('create_order'), self::ORDER_BEFORE);

        $this->setConfig('methods_updated_at', DateHelper::convertDateToString(new DateTime()));

        $sql= "CREATE TABLE IF NOT EXISTS `"._DB_PREFIX_."everypay_cards`(
        `id_card` INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY ,
        `card_no` VARCHAR(6) NOT NULL,
        `card_exp_year` VARCHAR(6) NOT NULL,
        `card_exp_month` VARCHAR(6) NOT NULL,
        `card_type` varchar(100) NOT NULL,
        `id_customer` int(11) NOT NULL,
        `token` VARCHAR(256) NOT NULL,
        `is_default` TINYINT(4) NOT NULL,
        `date_added` date NOT NULL
        )";

        Db::getInstance()->Execute($sql);
        $this->createEverypayPaymentsTable();
        $this->createPaymentStatusTable();

        return
            parent::install() &&
            $this->registerHook('paymentOptions') &&
            $this->registerHook('displayPaymentReturn') &&
            $this->registerHook('displayHeader') &&
            $this->registerHook('customerAccount') &&
            $this->registerHook('displayMyAccountBlock') &&
            $this->addOrderState($this->os_processing) &&
            $this->addOrderState($this->os_awaiting);
    }

    public function createEverypayPaymentsTable()
    {
        $sql = "CREATE TABLE IF NOT EXISTS `"._DB_PREFIX_."everypay_payments`(
        `order_reference` INT(11) NOT NULL,
        `payment_reference` VARCHAR(256) NOT NULL
        )";
        Db::getInstance()->execute($sql);
    }

    public function createPaymentStatusTable()
    {
        $sql = "CREATE TABLE IF NOT EXISTS `" . _DB_PREFIX_ . self::PAYMENT_STATUS_TABLE . "`(
            `id_status` INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,
            `order_reference` VARCHAR(256) NOT NULL,
            `status` VARCHAR(256) NOT NULL,
            INDEX `order_reference_index` (`order_reference`)
        )";
        Db::getInstance()->execute($sql);
    }

    public function uninstall()
    {
        Configuration::deleteByName($this->formatName('enabled_payment'));

        $this->removeConfig('methods_updated_at');

        return parent::uninstall();
    }

    public function getContent()
    {
        $output = '';
        $output .= $this->displayInfo();
        $output .= $this->postProcess();
        $output .= $this->displayForm();
        return $output;
    }

    protected function postProcess()
    {
        if (Tools::isSubmit('submitAddconfiguration')) {
            Configuration::updateValue($this->formatName('test_api_username'), Tools::getValue('test_api_username'));
            Configuration::updateValue($this->formatName('test_shared_secret'), Tools::getValue('test_shared_secret'));
            Configuration::updateValue($this->formatName('api_username'), Tools::getValue('api_username'));
            Configuration::updateValue($this->formatName('account_id'), Tools::getValue('account_id'));
            Configuration::updateValue($this->formatName('shared_secret'), Tools::getValue('shared_secret'));
            Configuration::updateValue($this->formatName('create_order'), Tools::getValue('create_order'));
            Configuration::updateValue($this->formatName('live_mode'), Tools::getValue('live_mode'));
            Configuration::updateValue($this->formatName('enabled_payment'), Tools::getValue('enabled_payment'));
//            Configuration::updateValue($this->formatName('theme'), Tools::getValue('theme'));
            Configuration::updateValue($this->formatName('layout'), Tools::getValue('layout'));
            Configuration::updateValue($this->formatName('default_country'), Tools::getValue('default_country'));

            $titles = [];
            $languages = Language::getLanguages(false);
            foreach ($languages as $lang) {
                if (Tools::getIsset('checkout_title_' . $lang['id_lang'])) {
                    $titles[$lang['id_lang']] = Tools::getValue('checkout_title_' . $lang['id_lang']);
                }
            }

            Configuration::updateValue($this->formatName('checkout_title'), $titles);
        }
    }

    public function displayInfo()
    {
        $this->smarty->assign(array(
            'path' => $this->getPathUri(),
            'everypay_link' => 'https://every-pay.com/',
            'everypay_doc' => 'https://support.every-pay.com/ ',
        ));
        if (Tools::version_compare(_PS_VERSION_, '1.6', '>')) {
            return $this->fetch('module:everypay/views/templates/hook/info.tpl');
        } else {
            return $this->display(__FILE__, 'info_15.tpl');
        }
    }

    public function displayForm()
    {
        $default_lang = (int)Configuration::get('PS_LANG_DEFAULT');

        $fields_form = array(array(
            'form' => array(
                'legend' => array('title' => $this->l('Settings')),
                'input' => array(
                    array(
                        'type'     => 'free',
                        'label'    => $this->l('Callback Notification URL'),
                        'name'     => 'free_string_name',
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('Checkout Title'),
                        'name'     => 'checkout_title',
                        'lang'     => true,
                    ),
                    array(
                        'type'     => 'radio',
                        'label'    => $this->l('Live Mode'),
                        'name'     => 'live_mode',
                        'required' => true,
                        'class'    => 't',
                        'values'   => array(
                            array('value' => 1, 'label' => $this->l('On'), 'id' => 'live_mode_on'),
                            array('value' => 0, 'label' => $this->l('Off'), 'id' => 'live_mode_off')
                        )
                    ),
//                    array(
//                        'type'     => 'select',
//                        'label'    => $this->l('Theme'),
//                        'name'     => 'theme',
//                        'required' => true,
//                        'class'    => 't',
//                        'options'  => [
//                            'query' => [
//                                [
//                                    'value' => self::THEME_LIGHT,
//                                    'label' => $this->l('Light'),
//                                    'id' => self::THEME_LIGHT,
//                                ],
//                                [
//                                    'value' => self::THEME_DARK,
//                                    'label' => $this->l('Dark'),
//                                    'id' => self::THEME_DARK,
//                                ],
//                            ],
//                            'id'    => 'value',
//                            'name'  => 'label',
//                        ],
//                    ),
                    array(
                        'type'     => 'select',
                        'label'    => $this->l('Layout'),
                        'name'     => 'layout',
                        'required' => true,
                        'class'    => 't',
                        'options'  => [
                            'query' => [
                                [
                                    'value' => self::LAYOUT_TAB,
                                    'label' => $this->l('Tab'),
                                    'id' => self::LAYOUT_TAB,
                                ],
                                [
                                    'value' => self::LAYOUT_ACCORDION,
                                    'label' => $this->l('Accordion'),
                                    'id' => self::LAYOUT_ACCORDION,
                                ],
                            ],
                            'id'    => 'value',
                            'name'  => 'label',
                        ]
                    ),
                    array(
                        'type'     => 'radio',
                        'label'    => $this->l('Create order'),
                        'name'     => 'create_order',
                        'required' => true,
                        'class'    => 't',
                        'desc'     => $this->l('create_order_desc'),
                        'values'   => array(
                            array(
                                'value' => self::ORDER_BEFORE,
                                'label' => $this->l('Before payment'),
                                'id'    => 'everypay_order_before'
                            ),
                            array(
                                'value' => self::ORDER_AFTER,
                                'label' => $this->l('After payment'),
                                'id'    => 'everypay_order_after'
                            )
                        )
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('Processing Account'),
                        'name'     => 'account_id',
                        'size'     => 50,
                        'required' => true,
                        'desc'     => $this->l('account_id_desc')
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('API Username'),
                        'name'     => 'api_username',
                        'size'     => 50,
                        'required' => true,
                        'desc'     => $this->l('api_username_desc')
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('API Secret'),
                        'name'     => 'shared_secret',
                        'size'     => 50,
                        'required' => true,
                        'desc'     => $this->l('shared_secret_desc'),
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('Test API Username'),
                        'name'     => 'test_api_username',
                        'size'     => 50,
                        'desc'     => $this->l('Only used for testing payments in the demo environment.
                            You can find this in the EveryPay demo merchant portal, under
                            “Settings/General settings/API username” (example: 49bcdd6fed983c12).'
                        )
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('Test API Secret'),
                        'name'     => 'test_shared_secret',
                        'size'     => 50,
                        'desc'     => $this->l('Only used for testing payments in the demo environment.
                            You can find this in the EveryPay demo merchant portal, under
                            “Settings/General settings/API secret” (example: e7ed4e55d5f73158f6cf2890fb1c950e).'
                        )
                    ),
                    array(
                        'type'     => 'select',
                        'label'    => $this->l('Default Country'),
                        'name'     => 'default_country',
                        'required' => true,
                        'class'    => 't',
                        'options'  => [
                            'query' => [
                                [
                                    'value' => self::COUNTRY_DEFAULT,
                                    'label' => $this->l('Default (by billing country)'),
                                    'id' => self::COUNTRY_DEFAULT,
                                ],
                                [
                                    'value' => self::COUNTRY_ESTONIA,
                                    'label' => $this->l('Estonia'),
                                    'id' => self::COUNTRY_ESTONIA,
                                ],
                                [
                                    'value' => self::COUNTRY_LATVIA,
                                    'label' => $this->l('Latvia'),
                                    'id' => self::COUNTRY_LATVIA,
                                ],
                                [
                                    'value' => self::COUNTRY_LITHUANIA,
                                    'label' => $this->l('Lithuania'),
                                    'id' => self::COUNTRY_LITHUANIA,
                                ],
                            ],
                            'id'    => 'value',
                            'name'  => 'label',
                        ],
                        'desc' => $this->l('By default, the country selection for the Open Banking payment method is determined by the selected billing country'),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                )
            )
        ));
        $helper_form = new HelperForm();
        $helper_form->module = $this;
        $helper_form->name_controller = $this->name;
        $helper_form->token = Tools::getAdminTokenLite('AdminModules');
        $helper_form->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;
        $helper_form->default_form_language = $default_lang;
        $helper_form->allow_employee_form_lang = $default_lang;
        $helper_form->languages = Language::getLanguages(false);
        $titles = [];
        foreach ($helper_form->languages as $k => $language) {
            $is_default = (int)($language['id_lang'] == $helper_form->default_form_language);
            $helper_form->languages[$k]['is_default'] = $is_default;

            $titles[$language['id_lang']] = $this->getTitle($language['id_lang']);
        }
        $helper_form->fields_value['test_api_username'] = Configuration::get($this->formatName('test_api_username'));
        $helper_form->fields_value['test_shared_secret'] = Configuration::get($this->formatName('test_shared_secret'));
        $helper_form->fields_value['api_username'] = Configuration::get($this->formatName('api_username'));
        $helper_form->fields_value['account_id'] = Configuration::get($this->formatName('account_id'));
        $helper_form->fields_value['shared_secret'] = Configuration::get($this->formatName('shared_secret'));
        $helper_form->fields_value['create_order'] = Configuration::get($this->formatName('create_order'));
        $helper_form->fields_value['live_mode'] = Configuration::get($this->formatName('live_mode'));
        $helper_form->fields_value['enabled_payment'] = Configuration::get($this->formatName('enabled_payment'));
//        $helper_form->fields_value['theme'] = Configuration::get($this->formatName('theme'));
        $helper_form->fields_value['layout'] = Configuration::get($this->formatName('layout'));
        $helper_form->fields_value['default_country'] = Configuration::get($this->formatName('default_country'));
        $helper_form->fields_value['checkout_title'] = $titles;

        $callbackInfo = $this->l('Ensure you enter the Callback Notification URL in the EveryPay merchant portal
            under “Settings/E-shop Settings” to sync payment information between your online store and EveryPay account. '
        );
        $callbackUrl = $this->context->link->getModuleLink($this->name,'return');

        $helper_form->fields_value['free_string_name'] = '<div style="padding-right: 20%; font-size: 12px ">'. $callbackInfo . '
        <br><br>
        Callback URL: <strong><i>'. $callbackUrl .'</i></strong></div>';

        return $helper_form->generateForm($fields_form);
    }

    /**
     * States if the live mode is enabled.
     *
     * @return false|string
     */
    public function isLiveMode() {
        return $this->getConfig('live_mode');
    }

    /**
     * Get the setting based on the environment.
     *
     * @param $setting
     * @return string|false
     */
    public function getCredentialSetting($setting) {
        if ($this->isLiveMode()) {
            $prefix = '';
        } else {
            $prefix = 'test_';
        }

        return $this->getConfig($prefix . $setting);
    }

    public function formatName($name = '')
    {
        return Tools::strtoupper($this->name.'_'.$name);
    }

    /**
     * Check if the order was created before payment.
     *
     * @return bool
     */
    public function isOrderCreatedBefore()
    {
        return Configuration::get($this->formatName('create_order')) === self::ORDER_BEFORE;
    }

    /**
     * Get a config value.
     *
     * @param $key
     * @return false|string
     */
    public function getConfig($key)
    {
        return Configuration::get($this->formatName((string)$key));
    }

    /**
     * Update a config value.
     *
     * @param $key
     * @param $value
     * @return void
     */
    public function setConfig($key, $value)
    {
        Configuration::updateValue($this->formatName((string)$key), $value);
    }

    /**
     * Remove a config value.
     *
     * @param $key
     * @return void
     */
    public function removeConfig($key)
    {
        Configuration::deleteByName($this->formatName((string)$key));
    }

    public function getEveryPayAPI()
    {
        if (!class_exists('EveryPayLibrary')) {
            include $this->local_path.'lib/everypay_library.php';
        }

        $api = null;
        $live_mode = $this->getConfig('live_mode');

        $username = $this->getConfig((!$live_mode ? 'test_' : '').'api_username');
        $secret = $this->getConfig((!$live_mode ? 'test_' : '').'shared_secret');

        if (!empty($username) && !empty($secret)) {
            $api = new EveryPayLibrary();
            $api->init($username, $secret);
        }

        return $api;
    }

    /**
     * Create Order State, coping mails and logo
     * @return boolean  true, if successful
     */
    public function addOrderState($stateArr)
    {
        $orderStateExist = false;
        $status_name = $stateArr['status_name'];
        $orderStateId = Configuration::get($status_name);
        $description = $stateArr['description'];
        $descriptionEE = $stateArr['descriptionEE'];
        if ($orderStateId) {
            $orderState = new OrderState($orderStateId);
            if ($orderState->id && !$orderState->deleted) {
                $orderStateExist = true;
            }
        } else {
            $query = 'SELECT os.`id_order_state` '.
                'FROM `%1$sorder_state_lang` osl '.
                'LEFT JOIN `%1$sorder_state` os '.
                'ON osl.`id_order_state`=os.`id_order_state` '.
                'WHERE osl.`name`="%2$s" OR osl.`name`="%3$s" AND os.`deleted`=0';

            $orderStateId =  Db::getInstance()->getValue(sprintf($query, _DB_PREFIX_, $description, $descriptionEE));
            if ($orderStateId) {
                Configuration::updateValue($status_name, $orderStateId);
                $orderStateExist = true;
            }
        }

        if (!$orderStateExist) {
            $languages = Language::getLanguages(false);
            $orderState = new OrderState();
            $estLangID = $this->getEstonianLangId() ?: false;

            foreach ($languages as $lang) {
                if ($lang['id_lang'] == $estLangID) {
                    $orderState->name[$lang['id_lang']] = $descriptionEE;
                } else {
                    $orderState->name[$lang['id_lang']] = $description;
                }
            }
            $orderState->send_email = 0;
            $orderState->invoice = 0;
            $orderState->color = $stateArr['color'];
            $orderState->unremovable = 0;
            $orderState->logable = 0;
            $orderState->delivery = 0;
            $orderState->hidden = 0;
            if ($orderState->add()) {
                Configuration::updateValue($status_name, $orderState->id);
                $orderStateExist = true;
            }
        }
        $file = $this->getLocalPath().'/views/img/everypay_status.gif';
        $newfile = _PS_IMG_DIR_.'os/' . $orderState->id . '.gif';
        copy($file, $newfile);

        return ($orderStateExist);
    }

    public function getOrderConfUrl($order)
    {
        return $this->context->link->getPageLink(
            'order-confirmation',
            true,
            $order->id_lang,
            array(
                'id_cart' => $order->id_cart,
                'id_module' => $this->id,
                'id_order' => $order->id,
                'key' => $order->secure_key
            )
        );
    }

    public function getOrderIdByReference($orderReference)
    {
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('orders');
        $sql->where('reference = '.'"'.$orderReference.'"');

        return Db::getInstance()->getValue($sql);
    }

    public function getOrderIdByCartId($cartId)
    {
        $sql = new DbQuery();
        $sql->select('id_order')
            ->from('orders')
            ->where('id_cart = '.(int)$cartId);

        return Db::getInstance()->getValue($sql);
    }

    /**
     * Get the payment status based on the order reference.
     *
     * @param string $orderReference
     *
     * @return false|string|null
     */
    public function getPaymentStatusByReference($orderReference)
    {
        $sql = new DbQuery();
        $sql->select('status');
        $sql->from(self::PAYMENT_STATUS_TABLE);
        $sql->where('order_reference = ' . '"' . $orderReference . '"');

        return Db::getInstance()->getValue($sql);
    }

    /**
     * Stores the payment status for an order.
     *
     * @param string $orderReference
     * @param string $paymentStatus
     *
     * @return void
     *
     * @throws PrestaShopDatabaseException
     */
    public function addOrderPaymentStatus($orderReference, $paymentStatus)
    {
        Db::getInstance()->insert(self::PAYMENT_STATUS_TABLE, [
            'order_reference' => pSQL($orderReference),
            'status' => pSQL($paymentStatus),
        ]);
    }

    /**
     * Updates the payment status for an order.
     *
     * @param string $orderReference
     * @param string $paymentStatus
     *
     * @return void
     */
    public function updateOrderPaymentStatus($orderReference, $paymentStatus)
    {
        \Db::getInstance()->update(
            self::PAYMENT_STATUS_TABLE,
            [
                'status' => pSQL($paymentStatus),
            ],
            'order_reference=' . pSQL($orderReference)
        );
    }

    /**
     * Set the get params, in case these are not present in the $_GET Superglobal.
     *
     * @return void
     */
    public function setGetParams()
    {
        if (!empty($_GET)) {
            return;
        }

        $parts = parse_url($_SERVER['REQUEST_URI']);
        parse_str($parts['query'], $queryParams);
        $_GET = $queryParams;
    }

    private function getLangCode()
    {
        $langCode = $this->context->language->iso_code;

        if (in_array(
            $langCode,
            ['cz', 'da', 'de', 'en', 'es', 'et', 'fi', 'fr', 'hu', 'it', 'lt', 'lv', 'nl',
                'no', 'pl', 'pt', 'ru', 'sk', 'sv', 'uk'
            ]
        )) {
            return $langCode;
        }

        return 'en';
    }

    private function getPreferredCountry()
    {
        $defaultCountry = $this->getConfig('default_country');
        $availableCountries = [self::COUNTRY_ESTONIA, self::COUNTRY_LATVIA, self::COUNTRY_LITHUANIA];
        if (in_array($defaultCountry, $availableCountries)) {
            return $defaultCountry;
        }

        $billingCountryId = $this->context->cart->id_address_invoice;
        if (!$billingCountryId) {
            return '';
        }

        $billingAddress = new \Address($billingCountryId);
        $billingCountry = new \Country($billingAddress->id_country);

        if (in_array($billingCountry->iso_code, $availableCountries)) {
            return $billingCountry->iso_code;
        }

        return '';
    }

    //Hooks START
    public function hookDisplayHeader($params)
    {
        if (Tools::getValue('controller') == 'order') {
            $everypayIdAddressDelivery = $this->context->cart->id_address_delivery;
            Media::addJsDef(['everypayIdAddressDelivery' => $everypayIdAddressDelivery]); // task 17971:
            // above passes delivery address id into JS, to get address country later
        }

        Media::addJsDef([
            'everypay_checkout_params' => [
                'account' => $this->getConfig('account_id'),
                'amount' => $this->context->cart->getOrderTotal(true, Cart::BOTH),
                'environment' => $this->isLiveMode() ? 'PRODUCTION' : 'demo',
                'layout' => $this->getConfig('layout') ?? self::LAYOUT_TAB,
                'locale' => $this->getLangCode(),
                'preferredCountry' => $this->getPreferredCountry(),
                'theme' => self::THEME_LIGHT,
                'username' => $this->getCredentialSetting('api_username'),
                'validationUrl' => $this->context->link->getModuleLink($this->name, 'payment', [], true),
            ],
        ]);

        $this->context->controller->registerJavascript(
            'everypay-script',
            'modules/' . $this->name . '/views/js/everypay.js'
        );

        if ($this->isLiveMode()) {
            $url = self::PAYMENT_ELEMENT_PROD_URL;
        } else {
            $url = self::PAYMENT_ELEMENT_TEST_URL;
        }

        $this->context->controller->registerJavascript(
            'everypay-payment-element',
            $url,
            ['server' => 'remote']
        );

        $this->context->controller->addCSS(($this->_path."views/css/").'everypay.css', 'all');
    }

    public function hookCustomerAccount($params)
    {
        return;

        $this->context->smarty->assign(array(
            'token_payment' => Configuration::get($this->formatName("token_payment")),
            'logged' => $this->context->customer->isLogged()
        ));
        return $this->fetch('module:everypay/views/templates/hook/everypay-myaccount.tpl');
    }

    public function hookDisplayMyAccountBlock($params)
    {
        return $this->hookCustomerAccount($params);
    }
    //Hooks END

    // New Payment Option set option visible on front pages START
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }

        if (!$this->checkCurrency($params['cart'])) {
            return;
        }

        $paymentOption = new PaymentOption();
        $paymentOption
            ->setCallToActionText($this->getTitle())
            ->setModuleName($this->name)
            ->setForm($this->context->smarty->fetch('module:everypay/views/templates/front/everypay.tpl'));

        return [$paymentOption];
    }

    public function getTitle($langId = null)
    {
        if ($langId === null) {
            $langId = $this->context->language->id;
        }

        $title = Configuration::get($this->formatName('checkout_title'), $langId);
        if (!empty($title)) {
            return $title;
        }

        if (Tools::getIsset('checkout_title_' . $langId)) {
            return Tools::getValue('checkout_title_' . $langId);
        }

        return $this->l('Online Payment');
    }

    // Payment Option inside form - end
    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);

        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }

    public function hookDisplayPaymentReturn($params)
    {
        $order = $params['order'];
        if ($params['order']) {
            $this->smarty->assign(array(
                'shop_name' => $this->context->shop->name,
                'total' => Tools::displayPrice(
                    $params['order']->getOrdersTotalPaid()
                )
            ));
            $status = 'ok';
        } else {
            $status = 'error';
        }

        $orderHistory = $this->context->link->getPageLink(
            'order-detail',
            true,
            $order->id_lang,
            array(
                'id_order' => $order->id
            )
        );

        $this->smarty->assign(array(
            'status' => $status,
            'linkToOrder' => $orderHistory
         ));

        return $this->fetch('module:everypay/views/templates/hook/payment-return.tpl');
    }

    public function getFormattedPaymentMethod($method = 'missing_payment_method') {
        // usage in controller:
        // $this->module->getFormattedPaymentMethod (Tools::getValue('payment_method') ? Tools::getValue('payment_method') : 'missing_payment_method'),

        $output = 'Everypay';
        switch ($method) {
            case 'apple_pay':
            case 'google_pay':
            case 'card' :
                $output = $this->l('Card payment', 'validation');
                break;
            case 'paypal' :
                $output = $this->l('Paypal payment', 'validation');
                break;
            case 'missing_payment_method':
            case '' :
                // these 2 provide module name as output
                break;
            default :
                // this covers various bank payment methods, eg: swed_ob_ee
                $output = $this->l('Bank payment', 'validation');
                break;
        }

        return $output;
    }

    protected function getEstonianLangId()
    {
        $sql = new DbQuery();
        $sql->select('id_lang');
        $sql->from('lang');
        $sql->where("iso_code = 'et'");
        $sql->where("active = 1");
        $results = Db::getInstance()->getValue($sql);
        return $results;
    }

    public function logError($errorObj, $identifier = '')
    {
        $log = date('d-m-Y H:i:s').' : ' . $identifier;
        $log .= "\n";
        if (!empty($errorObj)) {
            $errorText = print_r( $errorObj, true);
            $log .= $errorText;
            $log .= "\n";
        }

        $file = dirname(__FILE__).'/log/debug.log';

        file_put_contents($file, $log, FILE_APPEND | LOCK_EX);
    }

    public function insertOrderMessage($msgText, $order)
    {
        $customer = new Customer ($order->id_customer);

        $ct = new CustomerThread();
        $ct->id_contact = 0;
        $ct->id_customer = (int) $order->id_customer;
        $ct->id_shop = (int) $order->id_shop;
        $ct->id_order = (int) $order->id;
        $ct->id_lang = (int) $order->id_lang;
        $ct->email = $customer->email;
        $ct->status = 'closed';
        $ct->token = Tools::passwdGen(12);
        $ct->add();

        $cm = new CustomerMessage();
        $cm->id_customer_thread = $ct->id;
        $cm->message = $msgText;
        $cm->private = 1;
        $cm->add();
    }

    /**
     * Update the payment methods, if these have not been updated in the last day.
     * 
     * @return void
     */
    public function updatePaymentMethods()
    {
        $methodsUpdatedAt = $this->getConfig('methods_updated_at');
        if (empty($methodsUpdatedAt)) {
            $formattedDate = DateHelper::convertDateToString(new DateTime());

            $this->setConfig('methods_updated_at', $formattedDate);
            $methodsUpdatedAt = $formattedDate;
        }

        try {
            $date = (new DateTime($methodsUpdatedAt))->setTime(0, 0);
            $currentDate = (new DateTime())->setTime(0, 0);

            if (!($currentDate > $date) && $this->getConfig('methods')) {
                return;
            }

            if (!class_exists('AccountInfo')) {
                include $this->local_path . 'lib/AccountInfo.php';
            }

            $accountInfo = new AccountInfo();

            $this->setConfig('methods', $accountInfo->getFormatedPaymentMethods());
            $this->setConfig('methods_updated_at', DateHelper::convertDateToString(new DateTime()));
        } catch (Exception $e) {
            return;
        }
    }

    /**
     * Update the title of an order's payment method.
     * 
     * @param Order $order
     * @param string $paymentMethodID
     * @return void
     */
    public function updateOrderPaymentTitle($order, $paymentMethodID)
    {
        if (!$order || !$paymentMethodID) {
            return;
        }

        $newTitle = $this->getPaymentMethodTitle($paymentMethodID);
        if ($newTitle === '') {
            return;
        }

        try {
            $order->payment = $newTitle;
            $order->update();
        } catch (Exception $e) {}
    }

    /**
     * Get the payment method title based on the payment method ID.
     *
     * @param string $paymentMethodID
     * @return string
     */
    public function getPaymentMethodTitle($paymentMethodID)
    {
        $paymentMethods = json_decode($this->getConfig('methods'), true);
        if (!$paymentMethods) {
            return '';
        }

        foreach ($paymentMethods as $paymentMethod) {
            if ($paymentMethod['source'] === $paymentMethodID) {
                return $paymentMethod['name'];
            }
        }
        
        return '';
    }
}
