<?php
/**
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 *
 *  @author    EveryPay
 *  @copyright since 2015 EveryPay
 *  @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

class AccountInfo
{
    /**
     * @var array
     */
    private $info;

    /**
     * AccountInfo constructor.
     */
    public function __construct()
    {
        $this->hydrateAccountInfo();
    }

    /**
     * Hydrate the account info.
     *
     * @return void
     */
    public function hydrateAccountInfo()
    {
        if (!empty($this->info)) {
            return;
        }

        $isLiveMode = Configuration::get('EVERYPAY_LIVE_MODE');
        $apiUsername = $isLiveMode ? Configuration::get('EVERYPAY_API_USERNAME') : Configuration::get('EVERYPAY_TEST_API_USERNAME');
        $apiSecret = $isLiveMode ? Configuration::get('EVERYPAY_SHARED_SECRET') : Configuration::get('EVERYPAY_TEST_SHARED_SECRET');
        $accountInfoBaseUrl = $isLiveMode ? 'https://pay.every-pay.eu/api/v3/processing_accounts' : 'https://igw-demo.every-pay.com/api/v3/processing_accounts';
        $accountInfoUrl = $this->getPaymentAccountInfoUrl($accountInfoBaseUrl, Configuration::get('EVERYPAY_ACCOUNT_ID'), $apiUsername);
        $this->info = json_decode($this->getApiAccountInfo($accountInfoUrl, $apiUsername, $apiSecret), true);
    }

    /**
     * Get the account info from the API.
     *
     * @param string $url
     * @param string $username
     * @param string $password
     *
     * @return bool|string
     */
    private function getApiAccountInfo($url, $username, $password)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                'Content-Type: application/json'
            )
        );
        curl_setopt($ch, CURLOPT_USERPWD, $username . ':' . $password);
        $response = curl_exec($ch);
        curl_close($ch);

        return $response;
    }

    /**
     * Get the API payment account info URL.
     * 
     * @param string $baseUrl
     * @param string $accountName
     * @param string $apiUsername
     *
     * @return string
     */
    private function getPaymentAccountInfoUrl($baseUrl, $accountName, $apiUsername)
    {
        return $baseUrl . '/' . $accountName . '?api_username=' . $apiUsername;
    }

    /**
     * Get the payment methods in JSON format.
     *
     * @return false|string
     */
    public function getFormatedPaymentMethods()
    {
        $formattedPaymentMethods = [];

        foreach ($this->info['payment_methods'] as $paymentMethod) {
            $formatedMethod = [];
            $formatedMethod['source'] = $paymentMethod['source'];
            $formatedMethod['name'] = $paymentMethod['display_name'];
            $formatedMethod['country'] = $paymentMethod['country_code'];

            $formattedPaymentMethods[] = $formatedMethod;
        }

        return json_encode($formattedPaymentMethods);
    }
}
