/*
 * NOTICE OF LICENSE
 *
 * This file is licenced under the Software License Agreement.
 * With the purchase or the installation of the software in your application
 * you accept the licence agreement.
 *
 * You must not modify, adapt or create derivative works of this source code
 *
 *  @author    EveryPay
 *  @copyright since 2015 EveryPay
 *  @license   LICENSE.txt
 */
"use strict";
var EVERYPAY = {};
EVERYPAY.id = 'everypay';
EVERYPAY.paymentElement = '';
EVERYPAY.selectedPaymentMethod = '';

window.addEventListener('load', function() {
    EVERYPAY.init();
});

EVERYPAY.init = function () {
    EVERYPAY.loadPaymentElement();
    EVERYPAY.attachButtonClickEvent();
    EVERYPAY.selectedPaymentMethod = EVERYPAY.getSelectedPaymentMethod();
};

EVERYPAY.storeData = function(data) {
    if (!sessionStorage.getItem(EVERYPAY.id)) {
        sessionStorage.setItem(EVERYPAY.id, JSON.stringify({}));
    }
    const storedData = JSON.parse(sessionStorage.getItem(EVERYPAY.id));
    for (let dataKey in data) {
        storedData[dataKey] = data[dataKey];
    }

    sessionStorage.setItem(EVERYPAY.id, JSON.stringify(storedData));
}

EVERYPAY.getSelectedPaymentMethod = function() {
    const storedData = JSON.parse(sessionStorage.getItem(EVERYPAY.id));
    if (!storedData || !storedData.payment_method) {
        return 'card';
    }

    return storedData.payment_method;
}

EVERYPAY.loadPaymentElement = function() {
    const paymentElementDiv = document.querySelector(EVERYPAY.getPaymentElementSelector());
    if (!paymentElementDiv) {
        return;
    }

    const params = window.everypay_checkout_params;

    const environment = params.environment
    const account = params.account;
    const username = params.username;

    const everyPay = new window.EveryPay({
        account,
        username,
    });
    const stylingOptions = Object.freeze({
        theme: params.theme,
        layout: params.layout
    });
    const secureElementOptions = Object.freeze({
        stylingOptions,
        amount: parseFloat(params.amount),
        email: window.prestashop.customer.email,
        environment,
        locale: params.locale,
        preferredCountry: params.preferredCountry,
        username,
    });
    const paymentElementOptions = Object.freeze({
        element: 'payment',
    });

    let paymentElement = everyPay
        .secureElements(secureElementOptions)
        .build(paymentElementOptions);

    paymentElement.paymentMethod.addEventListener(
        'change',
        function addPaymentMethodNameToButton(e) {
            EVERYPAY.storeData({payment_method: e.detail.paymentMethod});
        }
    );

    paymentElement
        .mount(EVERYPAY.getPaymentElementSelector())
        .then(function () {
            EVERYPAY.paymentElement = paymentElement;
        })
        .catch(function onPaymentElementMountError(err) {
            console.error(err)
        });
}

EVERYPAY.getPaymentElementSelector = function() {
    return '#'+ EVERYPAY.id + 'PaymentElement';
}

// Get the selector for the 'Place Order' button.
EVERYPAY.getPlaceOrderButtonSelector = function() {
    return '#payment-confirmation button, #tc-payment-confirmation button';
}

EVERYPAY.disablePlaceOrderButton = function() {
    document.querySelector(EVERYPAY.getPlaceOrderButtonSelector()).disabled = true;
}

// 'Click' event for the PrestaShop default 'Place Order' button, when the Bank option is selected.
EVERYPAY.attachButtonClickEvent = function() {
    let placeOrderButton = $(EVERYPAY.getPlaceOrderButtonSelector());
    if (!placeOrderButton) {
        return;
    }

    placeOrderButton.on('click', async function(e) {
        if (EVERYPAY.isSelectedPaymentMethod()) {
            preventDefaultBehaviour(e);

            const submitResponse = await EVERYPAY.paymentElement.submit();
            if (!submitResponse.error) {
                $.ajax({
                    url: window.everypay_checkout_params.validationUrl,
                    data: {
                        'payment_method': EVERYPAY.getSelectedPaymentMethod()
                    },
                    method: 'POST',
                    success: async function(data) {
                        const response = await EVERYPAY.paymentElement.confirm(data.data)
                    }
                })
            }

            return;
        }

        function preventDefaultBehaviour(e) {
            e.preventDefault();
            e.stopImmediatePropagation();
        }
    });
}

EVERYPAY.isSelectedPaymentMethod = function() {
    return $(EVERYPAY.getPaymentElementSelector() + ':visible').length > 0;
}

// task 17972 - this restores everypay payment options visibility after they being hidden by PS if terms checkbox is not checked atm
EVERYPAY.attachRadioListeners = function () {
    var everypayRadioOptions = document.getElementsByName("payment-option");
    everypayRadioOptions.forEach((option) => {
        option.addEventListener("change", async function() {
                await EVERYPAY.sleep(100);
                EVERYPAY.enableLinks();}
            ,false);
    });
};

// task 17972
EVERYPAY.sleep = function (ms) {
    return new Promise(resolve => setTimeout(resolve, ms));
};

// task 17972
EVERYPAY.areTermsAccepted = function () {
    let checkBoxStatus = document.getElementById('conditions_to_approve[terms-and-conditions]');
    if (!checkBoxStatus) {
        return false;
    } else {
        return checkBoxStatus.checked;
    }
};
